#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8

'''
  Intuition app entry point
  --------------------------

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

import sys
from intuition.cli import main


if __name__ == '__main__':
    sys.exit(main())
