'''
  Intuition
  ---------

  Intuition is an engine, some building bricks and a set of tools meant to let
  you efficiently and intuitively make your own automated quantitative trading
  system. It is designed to let traders, developers and scientists explore,
  improve and deploy market technical hacks.

  :copyright (c) 2014 Xavier Bruhiere
  :license: Apache 2.0, see LICENSE for more details.
'''

__project__ = 'intuition'
__author__ = 'Xavier Bruhiere'
__copyright__ = 'Xavier Bruhiere'
__licence__ = 'Apache 2.0'
__version__ = '0.4.2'
